#ifndef __KYXSET_H__
#define __KYXSET_H__
#include <X11/extensions/Xrandr.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Xlibint.h>

/*Error code*/

#define UNABLED_TO_FIND_DEVICEID 201           //无效的触摸设备ID
#define NOFIND_OUTPUT_NAME 202                 //无效的屏幕名称
#define INVAILD_RELATION_POS_VALUE 203         //无效的扩展屏相对位置
#define INVAILD_RATE_VALUE 204                 //无效的刷新率
#define INVAILD_MODE_VALUE 205                 //无效的分辨率
#define INVAILD_ROTATE_VALUE 206               //无效的旋转方向
#define OUTPUT_NAME_HAS_NO_MODES 207           //屏幕无此分辨率模式
#define NOFIND_CRTC 208                        //未知的CRTC
#define OUTPUT_CANNOT_USE_CRTC 209             //当前屏幕不能使用该CRTC
#define SCREEN_SIZE_LARGER_THAN_DESIRED 210    //分辨率超出最大值
#define SCREEN_SIZE_LESS_THAN_LEAST 211        //分辨率低于最小值
#define GET_SCREEN_RESOURCES_FAILED 212        //无法获取屏幕信息
#define GET_CRTC_INFORMATION_FAILED 213        //无法获取CRTC信息
#define GET_OUTPUT_INFORMATION_FAILED 214      //无法获取显示输出信息


#ifdef HAVE_USLEEP
# define Usleep(us) usleep((us))
#elif defined(WIN32)
# define Usleep(us) Sleep(us)
#else
# define Usleep(us) sleep((us / 1000000 > 0) ? us / 1000000 : 1)
#endif /* HAVE_USLEEP */


typedef enum _relation {
    relation_left_of,
    relation_right_of,
    relation_above,
    relation_below,
    relation_same_as,
} relation_t;

Display *OpenDisplay();
/* 

retrun                   If successful, returns a pointer to the connection to the X server
                         If OpenDisplay does not succeed, it returns NULL.

*/

int CloseDisplay(Display *dpy);
/*
Function 
    closes the connection to the X server 

Parameters

    dpy                   Specifies the connection to the X server.

*/

void DisableAccessControl(Display *dpy);
/*
Function 
    disables the use of the access control list at each connection setup. 

    For this functions to execute successfully, the client application must reside on 
    the same host as the X server and/or have been given permission in the initial authorization 
    at connection setup. 

Parameters

    dpy                   Specifies the connection to the X server.

*/

void EnableAccessControl(Display* dpy);
/*
Function 
    enables the use of the access control list at each connection setup.
    
    For this functions to execute successfully, the client application must reside on 
    the same host as the X server and/or have been given permission in the initial authorization 
    at connection setup. 

Parameters

    dpy                   Specifies the connection to the X server.

*/

int GetDefaultScreen(Display *dpy);
/*
This function should be used to retrieve the screen number in applications that will use only a single screen. 

Parameters

    dpy                   Specifies the connection to the X server.

return                    the default screen number   

*/

Window GetRootWindow(Display *dpy, int screen_number);
/*

Parameters

    dpy                   Specifies the connection to the X server.
    screen_number         Specifies the appropriate screen number on the host server. 

return                    the root window. 
*/

int GetDisplayWidth(Display *dpy, int screen_number);
/*

Parameters

    dpy                   Specifies the connection to the X server.
    screen_number         Specifies the appropriate screen number on the host server. 

return                    the width of the screen in pixels.
*/

int GetDisplayHeight(Display *dpy, int screen_number);
/*

Parameters

    dpy                    Specifies the connection to the X server.
    screen_number          Specifies the appropriate screen number on the host server. 

return                     the height of the screen in pixels.
*/

int  GetCrtcSize(char *name, int *Crtc_width, int *Crtc_height);
/* 
Funtion
    Used to obtain size of screen

Parameters

    name                   output name or OptType
    Crtc_width             obtain the width of the crtc of output name (or OptType)
    Crtc_height            obtain the height of the crtc of output name (or OptType)

return                     if successful return 0; 
                           else return -1;
*/

int ExternMonitorSet(Display *dpy, char *output_name_primary , 
                     char *output_name_extern ,relation_t relation_pos );
/*
Funtion
    Used to open the expansion screen and set the rotation of the expansion screen

Parameters

    dpy                    Specifies the connection to the X server.
    output_name_primary    output name used to be primary screen
    output_name_extern     output name used to be extern screen
    relation_pos           Specify the positional relation 

return                     if successful return 0; 
                           else return -1;
*/

int GetConnectedOutputCount(Display *dpy,Window window);
/*
Funtion
    Used to obtain the numbers of output which has been connected

Parameters

    dpy                    Specifies the connection to the X server.
    window                 the root window 

return                     Count of output which has been connected
*/

int GetConnectedOutputType(Display *dpy,Window window,char *OptType[]);
/*
Funtion
    Used to obtain the type of output which has been connected

Parameters

    dpy                    Specifies the connection to the X server.
    window                 the root window 
    OptType                obtain the type of output which has been connected
    
return                     if successful return 0; 
                           else return -1;
*/

Status SetScreenConfigAndRate (Display *dpy,Drawable draw,int width,int height,
                               char *orientation,double rate);
/*
Funtion
    Used to set the Config and Rate of the Specified screen,such as size 、rotate orientation

Parameters

    dpy                    Specifies the connection to the X server.
    draw                   Specifies which screen the Config and Rate is setted.
    width
	height                 Specify the width and height of screen size
    orientation            Specify the rotate orientation ("normal","inverted","left","right")
    rate                   Specify Screen refresh rate 

return                     if successful return 0; 
                           else return -1;
*/

int MonitorModeSet(Display *dpy, char *spec_output_name,char *spec_mode );
/*
Funtion
    Used to set the mode(size) of the Specified screen

Parameters

    dpy                    Specifies the connection to the X server.
    spec_output_name       Specifies which output is setted.
    spec_mode              Specify Screen mode(size)

return                     if successful return 0; 
                           else return -1;
*/

int MonitorRotateSet(Display *dpy, char *spec_output_name,char *orientation );
/*
Funtion
    Used to set the rotate orientation of the Specified screen 

Parameters

    dpy                    Specifies the connection to the X server.
    spec_output_name       Specifies which output is setted.
    orientation            Specify Screen rotate orientation

return                     if successful return 0; 
                           else return -1;
*/

int MonitorRateSet(Display *dpy, char *spec_output_name,double spec_Rate );
/*
Funtion
    Used to set the refresh rate of the Specified screen 
Parameters

    dpy                    Specifies the connection to the X server.
    spec_output_name       Specifies which output is setted.
    spec_Rate              Specify Screen refresh rate

return                     if successful return 0; 
                           else return -1;
*/

int MapToOutput(Display *dpy, char *name, char *outputname);
/*
Funtion
    Used to correct touch screen split offset

Parameters

    dpy                    Specifies the connection to the X server.
    name                   name or ID of touch
    outputname             output name or OptType

return                     if successful return 0; 
                           else return -1;
*/

void SetTouchWakeup(Display *dpy);
/*
Parameters

    dpy                    Specifies the connection to the X server.

Funtion
    Used to setting monitor wakeup after touching
*/

void SetDpmsState(Bool dpms_state);

/*
Funtion
    Used to setting display power off

Parameters
    dpms_state             set display power state,
                           0 : display power ON;
                           1 : display power OFF; 
*/



#endif